#ifndef STREAMNODE_HPP__
#define STREAMNODE_HPP__

#include "nodes/base/base.hpp"
#include <opencv2/opencv.hpp>

namespace Node
{

class StreamNode : public BaseNode
{
public:
    StreamNode() = delete;
    StreamNode(const std::string& name) : BaseNode(name, NODE_TYPE::SRC_NODE) {}
    StreamNode(const std::string& name, const std::string& url) : BaseNode(name, NODE_TYPE::SRC_NODE), stream_url_(url){}
    virtual ~StreamNode() { };

    void set_stream_url(const std::string& stream_url)
    {
        stream_url_ = stream_url;
    }

    void set_skip_frame(int skip_frame)
    {
        skip_frame_ = skip_frame;
    }

    void work() override
    {
        cv::VideoCapture cap(stream_url_);
        if (!cap.isOpened())
        {
            std::cerr << "Error: cannot open camera" << std::endl;
            return;
        }

        while (running_)
        {
            cv::Mat frame;
            cap >> frame;
            if (frame.empty())
            {
                std::cerr << "Error: cannot read frame" << std::endl;
                break;
            }

            auto metaData = std::make_shared<meta::MetaData>();
            metaData->image = frame;

            for (auto& output_buffer : output_buffers_)
            {
                output_buffer.second.push(metaData);
            }
        }
    }

private:
    std::string stream_url_;
    int skip_frame_ = 0;
};

}

#endif  // STREAMNODE_HPP__